ad_page_contract {
    {mail_job_id:integer,notnull}
    {bind_var:array,optional}
} {
}

permission::require_permission -object_id $class_id -privilege read
if {[exists_and_not_null list_id]} {
    db_1row get_list_name {}
    set list_url "lists?[export_url_vars list_id]"
    permission::require_permission -object_id $list_id -privilege read
} else {
    db_1row get_selection_title {}
    set selection_url "[mailing_list::util::get_sw_url -package_key users-selection]selection?[export_url_vars selection_id]"
    permission::require_permission -object_id $selection_id -privilege read
}

set context_bar [list [list "." Administration] "Preview Mail Job"]

db_1row get_default_data {}

if {![info exists text_body]} {
    set text_header ""
    set text_body ""
    set text_footer ""
}
if {![info exists html_body]} {
    set html_header ""
    set html_body ""
    set html_footer ""
}

if {$subject_change_p == "f"} {
    set subject $default_subject
}
if {$text_header_change_p == "f"} {
    set text_header $default_text_header
}
if {$text_body_change_p == "f"} {
    set text_body $default_text_body
}
if {$text_footer_change_p == "f"} {
    set text_footer $default_text_footer
}
if {$html_header_change_p == "f"} {
    set html_header $default_html_header
}
if {$html_body_change_p == "f"} {
    set html_body $default_html_body
}
if {$html_footer_change_p == "f"} {
    set html_footer $default_html_footer
}

set mail_subject $subject
if {![empty_string_p $text_body]} {
    set mail_text_body [ad_text_to_html -- "$text_header\n$text_body\n$text_footer"]
} else {
    set mail_text_body ""
}

if {![empty_string_p $html_body]} {
    set mail_html_body "$html_header\n$html_body\n$html_footer"
} else {
    set mail_html_body ""
}

set tcl_code_p [mailing_list::util::text_contains_tcl_code -text "$subject $text_header $text_body $text_footer $html_header $html_body $html_footer"]

template::multirow create attachments name

if {![db_0or1row get_old_css_file {}]} {
    db_0or1row get_current_css_file {}
}

if {[info exists css_file]} {
    template::multirow append attachments $css_file
}

db_foreach get_attachments {} {
    template::multirow append attachments $file_name
}

if {[regexp [template::adp_variable_regexp] "$subject $mail_text_body $mail_html_body"] || [regexp [template::adp_variable_regexp_noquote] $mail_html_body]} {
    set template_p 1
} else {
    set template_p 0
}

set form [ns_getform]
for { set i 0 } { $i < [ns_set size $form] } { incr i } {
    if {[regexp {bind_var\.(.*)} [ns_set key $form $i] match var_name]} {
	set bind_var($var_name) [ns_set value $form $i]
    }
}

template::multirow create var_errors var

if {$template_p} {
    # check if all used variables are valid
    set check_text "$subject $mail_text_body $mail_html_body"
    # strip out all noquotes
    while {[regsub -all [template::adp_variable_regexp_noquote] $check_text {\1@\2@} check_text]} {}

    # get list of all valid vars
    if {[exists_and_not_null list_id]} {
	# get all valid variables for mailing lists
	set valid_vars [list user_id user_first_names user_last_name user_email link view_mail_link]
    } else {
	# get sql-query
	if {![db_0or1row get_mail_job_sql_query {}]} {
	    db_1row get_users_selection_query {}
	}
	# resolve sql-query if bind vars are used
	if {[info exists bind_var]} {
	    set sql_query [db_bind_var_substitution $sql_query [array get bind_var]]
	}
	# get query columns - these are the variables names
	# the user is allowed to use
	set valid_vars [user_selection::get_field_names -query $sql_query]
	lappend valid_vars link view_mail_link
    }

    # check text if all used variables are valid
    set invalid_vars [mailing_list::util::check_valid_vars -text $check_text -valid_vars $valid_vars]

    # generate error messages for each invalid variable
    foreach invalid_var $invalid_vars {
	template::multirow append var_errors $invalid_var
    }
}

if {$template_p && [exists_and_not_null list_id] && [empty_string_p $invalid_vars]} {
    # variable found in text, so try to substitute variables

    set mail_subject $subject
    # we have to use noquote-substitution for mail subject
    while {[regsub -all [template::adp_variable_regexp] $mail_subject {\1@\2;noquote@} mail_subject]} {}
    if {![empty_string_p $mail_text_body]} {
	# we have to use noquote-substitution for plaintext emails
	while {[regsub -all [template::adp_variable_regexp] $mail_text_body {\1@\2;noquote@} mail_text_body]} {}
    }
    if {![empty_string_p $mail_html_body]} {
	# we have to use noquote-substitution for mail view tracking in html mails
	while {[regsub -all -nocase {@view_mail_link@} $mail_html_body {@view_mail_link;noquote@} mail_html_body]} {}
	while {[regsub -all -nocase {</?(html|body|head|title)[^>]*?>} $mail_html_body {} mail_html_body]} {}
    }

    set user_first_names "Joe"
    set user_last_name "User"
    set user_id 999
    set user_email "joe.user@hotmail.com"
    set link "[ad_url][ad_conn package_url]?user_id=$user_id&x=some_x_test"
    set view_mail_link "<img src=\"[ad_url][mailing_list::util::get_sw_url -package_key mail-links]pixel.gif\" width=\"1\" heigth=\"1\">"

    variable ::template::parse_level
    lappend ::template::parse_level [info level]
    # Now verify that the substitutions work
    with_catch __errmsg {
	if {![empty_string_p $mail_text_body]} {
	    set __result [template::adp_compile -string $mail_text_body]
	    template::adp_eval __result
	    set mail_text_body $__adp_output
	}
	if {![empty_string_p $mail_html_body]} {
	    set __result [template::adp_compile -string $mail_html_body]
	    template::adp_eval __result
	    set mail_html_body $__adp_output
	}
	set __result [template::adp_compile -string $subject]
	template::adp_eval __result
	set mail_subject $__adp_output
    } {
	# substitution failed, so write error-text in mail_body
	set mail_subject $subject
	set mail_text_body $__errmsg
    }
    template::util::lpop ::template::parse_level
}

ad_return_template
