ad_page_contract {
} {
    mail_job_id:integer,optional
    class_id:integer
    {list_id:integer,optional ""}
    {selection_id:integer,optional ""}
    delete_files:optional,multiple
    bind_var:array,optional
}

ns_log debug "jobs: $list_id and class_id=$class_id"

set user_id [auth::require_login]
set package_id [ad_conn package_id]
permission::require_permission -object_id $package_id -privilege mailing_list_admin
permission::require_permission -object_id $class_id -privilege read
if {[exists_and_not_null list_id]} {
    permission::require_permission -object_id $list_id -privilege read
}
if {[exists_and_not_null selection_id]} {
    permission::require_permission -object_id $selection_id -privilege read
    db_1row get_users_selection_query {}
}

set format "YYYY-MM-DD HH24:MI"

if {[info exists mail_job_id]} {
    db_1row check_if_job_exists {}
} else {
    set edit_job_p 0
}


if {$edit_job_p} {
    set action Edit
    set attachments [db_list_of_lists get_attachments {}]
    db_0or1row get_old_css_file {}
    db_0or1row get_old_list_or_users_selection {}
} else {
    set action Add
    set attachments ""
    db_0or1row get_current_css_file {}
}


set context_bar [list [list "." Administration] "$action Mail Job"]

set languages [db_list_of_lists get_ad_locales {}]

db_1row check_if_data_changeable {}

if {$mime_type == "text/plain"} {
    set default_html_header ""
    set default_html_body ""
    set default_html_footer ""
    set html_header_change_p f
    set html_body_change_p f
    set html_footer_change_p f
}

if {$mime_type == "text/html"} {
    set default_text_header ""
    set default_text_body ""
    set default_text_footer ""
    set text_header_change_p f
    set text_body_change_p f
    set text_footer_change_p f
}

ad_form -name mail_job_form -html {enctype multipart/form-data} -form {
    {mail_job_id:key}
    {class_id:text(hidden)}
    {class_name:text(inform) {label "Mail Class"}}
}

if {[exists_and_not_null list_id]} {
    # spamming a mailing list
    ad_form -extend -name mail_job_form -form {
	{list_id:text(hidden)}
	{list_name:text(inform) {label "Mailing List"}}
    }
    set allowed_variables [list user_id user_first_names user_last_name user_email link view_mail_link]
} else {
    # spamming an user-selection
    ad_form -extend -name mail_job_form -form {
	{selection_id:text(hidden)}
	{selection_name:text(inform) {label "Users Selection"}}
    }

    if {$edit_job_p} {
	set bind_vars_values [db_list_of_lists get_mail_job_bind_vars {}]
    } else {
	set bind_vars_values [db_list_of_lists get_selection_bind_vars {}]
    }

    # add form entry for each query bind variable
    set ad_form_code "-form \{\n"
    set subs_list ""
    foreach bind_value $bind_vars_values {
	util_unlist $bind_value var_name var_description var_value
	lappend subs_list $var_name $var_value
	append ad_form_code "\{bind_var.$var_name:text \{label \"$var_description\"\} \{value \"$var_value\"\}\}\n"
    }
    append ad_form_code "\}"
    eval ad_form -extend -name mail_job_form $ad_form_code

    if {![empty_string_p $subs_list]} {
	set query_string [db_bind_var_substitution $query_string $subs_list]
    }
    set allowed_variables [user_selection::get_field_names -query $query_string]
    lappend allowed_variables link view_mail_link
}

ad_form -extend -name mail_job_form -form {
    {locale:text(select) {label "Language"} {options $languages}}
    {sender_email:text {label "Sender Email"} {html {size 80 maxlength 1000}}}
}

set pretty_allowed_vars "@[join $allowed_variables "@, @"]@"
ad_form -extend -name mail_job_form -form {
    {zz:text(inform) {label " "} {value "You are allowed to use the following variables in the mail text: $pretty_allowed_vars"}}
}

if {$subject_change_p == "t"} {
    ad_form -extend -name mail_job_form -form {
	{subject:text {label "Subject"} {html {size 80 maxlength 1000}}}
    }
} else {
    ad_form -extend -name mail_job_form -form {
	{subject:text(inform) {label "Subject"}}
    }
}

if {$mime_type != "text/html"} {
    if {$text_header_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {text_header:text(textarea),optional {label "Text Header"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {text_header:text(inform),optional {label "Text Header"}}
	}
    }

    if {$text_body_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {text_body:text(textarea) {label "Text Body"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {text_body:text(inform) {label "Text Body"}}
	}
    }

    if {$text_footer_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {text_footer:text(textarea),optional {label "Text Footer"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {text_footer:text(inform),optional {label "Text Footer"}}
	}
    }
}

if {$mime_type != "text/plain"} {
    if {$html_header_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {html_header:text(textarea),optional {label "HTML Header"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {html_header:text(inform),optional {label "HTML Header"}}
	}
    }

    if {$html_body_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {html_body:text(textarea) {label "HTML Body"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {html_body:text(inform) {label "HTML Body"}}
	}
    }

    if {$html_footer_change_p == "t"} {
	ad_form -extend -name mail_job_form -form {
	    {html_footer:text(textarea),optional {label "HTML Footer"} {html {rows 5 cols 80}}}
	}
    } else {
	ad_form -extend -name mail_job_form -form {
	    {html_footer:text(inform),optional {label "HTML Footer"}}
	}
    }
}

if {[exists_and_not_null css_file]} {
    ad_form -extend -name mail_job_form -form {
	{css_file:text(inform) {label "CSS-File"}}
    }
}

if {[exists_and_not_null attachments]} {
    ad_form -extend -name mail_job_form -form {
	{delete_files:integer(checkbox),multiple,optional {label "Delete Attachments"} {options $attachments}}
    }
}

set ad_form_code { -form {
    {upload_file:file,optional {label "Attachment"}}
    {xx:text(inform) {label " "} {value "You can attach as many files as you want"}}
    {track_links_p:text(radio) {label "Track Links?"} {options {{"Yes" t} {"No" f}}}}
    {state:text(radio) {label "Status"} {options {{"Active" active} {"Suspended" suspended}}}}
    {yy:text(inform) {label " "} {value "Don't forget to activate after uploading all attachments"}}
    {scheduled_date:date,to_sql(sql_date),to_html(display_date) {label "Execution Date"} {format $format} {help}}
} -new_request {
    if {![db_0or1row get_mail_class_data {}]} {
	ad_return_complaint 1 "This mail class has been deleted."
	return
    }

    if {![empty_string_p $class_comment]} {
	set class_name "$class_name ($class_comment)"
    }
    if {![empty_string_p $list_id]} {
	db_1row get_list_name {}

	if {![empty_string_p $list_comment]} {
	    set list_name "$list_name ($list_comment)"
	}
    }
    if {![empty_string_p $selection_id]} {
	db_1row get_users_selection_name {}

	if {![empty_string_p $selection_desc]} {
	    set selection_name "$selection_name ($selection_desc)"
	}
    }

    set track_links_p t
    set state active
    set scheduled_date [util::date::now]

    if {$subject_change_p == "f"} {
	set subject [ad_quotehtml $subject]
    }
    if {$text_header_change_p == "f"} {
	set text_header [ad_quotehtml $text_header]
    }
    if {$text_body_change_p == "f"} {
	set text_body [ad_quotehtml $text_body]
    }
    if {$text_footer_change_p == "f"} {
	set text_footer [ad_quotehtml $text_footer]
    }
    if {$html_header_change_p == "f"} {
	set html_header [ad_quotehtml $html_header]
    }
    if {$html_body_change_p == "f"} {
	set html_body [ad_quotehtml $html_body]
    }
    if {$html_footer_change_p == "f"} {
	set html_footer [ad_quotehtml $html_footer]
    }
} -edit_request {
    permission::require_permission -object_id $mail_job_id -privilege admin

    if {![db_0or1row get_mail_job_data {}]} {
	ad_return_complaint 1 "This mail job has been deleted."
	return
    }

    set scheduled_date [util::date::acquire clock [clock scan $scheduled_date]]

    if {![empty_string_p $class_comment]} {
	set class_name "$class_name ($class_comment)"
    }
    if {![empty_string_p $list_comment]} {
	set list_name "$list_name ($list_comment)"
    }
    if {![empty_string_p $selection_desc]} {
	set selection_name "$selection_name ($selection_desc)"
    }
    if {$subject_change_p == "f"} {
	set subject [ad_quotehtml $subject]
    }
    if {$text_header_change_p == "f"} {
	set text_header [ad_quotehtml $text_header]
    }
    if {$text_body_change_p == "f"} {
	set text_body [ad_quotehtml $text_body]
    }
    if {$text_footer_change_p == "f"} {
	set text_footer [ad_quotehtml $text_footer]
    }
    if {$html_header_change_p == "f"} {
	set html_header [ad_quotehtml $html_header]
    }
    if {$html_body_change_p == "f"} {
	set html_body [ad_quotehtml $html_body]
    }
    if {$html_footer_change_p == "f"} {
	set html_footer [ad_quotehtml $html_footer]
    }
} -on_submit {
    permission::require_permission -object_id $class_id -privilege read
    if {[exists_and_not_null list_id]} {
	permission::require_permission -object_id $list_id -privilege read
    } else {
	permission::require_permission -object_id $selection_id -privilege read
    }

    if {![empty_string_p $upload_file]} {
	# if user uploaded a file, let him upload some more
	# and in the meantime mark created job as suspended
	set state "suspended"
    }
    if {$subject_change_p == "f"} {
	set subject $default_subject
    }
    if {$text_header_change_p == "f"} {
	set text_header $default_text_header
    }
    if {$text_body_change_p == "f"} {
	set text_body $default_text_body
    }
    if {$text_footer_change_p == "f"} {
	set text_footer $default_text_footer
    }
    if {$html_header_change_p == "f"} {
	set html_header $default_html_header
    }
    if {$html_body_change_p == "f"} {
	set html_body $default_html_body
    }
    if {$html_footer_change_p == "f"} {
	set html_footer $default_html_footer
    }

    if {[mailing_list::util::text_contains_tcl_code -text "$subject $text_header $text_body $text_footer $html_header $html_body $html_footer"]} {
	ad_return_error "Mail Text contains inline tcl-code" "It is not permitted to use inline tcl code in emails"
	return
    }

    if {[regexp [template::adp_variable_regexp] "$subject $text_header $text_body $text_footer $html_header $html_body $html_footer"] || [regexp [template::adp_variable_regexp_noquote] "$html_header $html_body $html_footer"]} {
	# variable found in text, so enable variable substitution when
	# sending email
	set template_p t
    } else {
	# no variable found, so don't try to substitute variables when
	# sending email (faster)
	set template_p f
    }
    if {[info exists bind_var]} {
	set bind_vars [array get bind_var]
    } else {
	set bind_vars ""
    }
} -new_data {
    set mail_job_id [mail_job::add -class_id $class_id -list_id $list_id \
			 -selection_id $selection_id -locale $locale \
			 -sender_email $sender_email -track_links_p $track_links_p \
			 -subject $subject -text_header $text_header -text_body $text_body \
			 -text_footer $text_footer -html_header $html_header \
			 -html_body $html_body -html_footer $html_footer \
			 -mime_type $mime_type -template_p $template_p -state $state \
			 -scheduled_date $scheduled_date -bind_vars $bind_vars]
} -edit_data {
    permission::require_permission -object_id $mail_job_id -privilege admin

    mail_job::edit -mail_job_id $mail_job_id -locale $locale -sender_email $sender_email \
	-track_links_p $track_links_p -subject $subject -text_header $text_header \
	-text_body $text_body -text_footer $text_footer -html_header $html_header \
	-html_body $html_body -html_footer $html_footer -mime_type $mime_type \
	-template_p $template_p -state $state -scheduled_date $scheduled_date \
	-bind_vars $bind_vars
} -after_submit {
    db_transaction {
	if {[info exists bind_var]} {
	    foreach var_name [array names bin_var] {
		set value $bind_var($var_name)
		db_dml update_bind_var {}
	    }
	}
	set target_url "."
	if {![empty_string_p $upload_file]} {
	    set filename [lindex $upload_file 0]
	    set tmp_filename [lindex $upload_file 1]
	    set file_mimetype [lindex $upload_file 2]
	    set n_bytes [file size $tmp_filename]
	    set max_file_size [ad_parameter MaxAttachmentSize]

	    if { $n_bytes > $max_file_size && $max_file_size > 0 } {
		ad_return_complaint 1 "Your file is too large.  The publisher of
                 [ad_system_name] has chosen to limit attachments to
                 [util_commify_number $max_file_size] bytes.\n"
		return
	    }
	    if { $n_bytes == 0 } {
		ad_return_complaint 1 "Your file is zero-length.  Either you attempted to
                 upload a zero length file, a file which does not exists, 
                 or something went wrong during the transfer.\n"
		return
	    }
	    cr_import_content -title $filename $mail_job_id $tmp_filename $n_bytes $file_mimetype $filename
	    set target_url "jobs?[export_url_vars mail_job_id class_id]"
	}
	if {[info exists delete_files]} {
	    foreach file_id $delete_files {
		db_exec_plsql delete_file {}

		set path "[cr_fs_path][cr_create_content_file_path $file_id ""]"
		foreach revision [glob -directory $path "*"] {
		    ns_unlink $revision
		}
		ns_rmdir $path
	    }
	}
    }

    ad_returnredirect $target_url
    ad_script_abort
}}

set preview_switch "-confirm_template job-preview"

array set formvars [util_ns_set_to_list -set [ns_getform]]
if {[info exists formvars(upload_file)] && [llength $formvars(upload_file)] > 0} {
    # show the same form if user uploads several files
    # only show confirmation page if user didn't upload another file
    set preview_switch ""
}

eval ad_form -extend -name mail_job_form $preview_switch -action jobs $ad_form_code

ad_return_template
